/* msDistri - microscope distributor
 * version 0.1 date 2009-06-19
 * Copyright (C) 2009 Stefan Schuermans <stefan@blinkenarea.org>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 * a BlinkenArea project - http://www.blinkenarea.org/
 */

#include <avr/interrupt.h>
#include <avr/io.h>

#include "config.h"
#include "macros.h"

// new value read from the switches
static unsigned short ConfigNewValue = 0;
// number of times the new value has been read
#define ConfigNewCntMax 50
static unsigned char ConfigNewCnt = 0;

// current configuration value
static unsigned short ConfigValue = 0;

// demo mode active
char ConfigDemoActive = 0; // boolean flag if demo mode is active (extern)

// MAC address
unsigned char ConfigMac[6] = { 0x02, 0x66, 0x70, 0x44, 0x00, 0x00 }; // (extern)

// IP configuration
unsigned char ConfigIp[4] = { 10, 23, 80, 0 }; // own IP address (extern)
unsigned char ConfigMask[4] = { 255, 255, 0, 0 }; // subnet mask (extern)
unsigned char ConfigGw[4] = { 10, 23, 0, 1 }; // gateway IP address (extern)

// read switches
static unsigned short ConfigRead( void )
{
  unsigned char sreg;
  unsigned short value;

  // turn off interrupts
  sreg = SREG;
  cli( );

  // read address
  value = (unsigned short)(unsigned char)~PIND | (unsigned short)(unsigned char)(~PINB >> 4 & 0x0F) << 8;

  // restore interrupt state
  SREG = sreg;

  return value;
}

// apply new configuration value
static void ConfigApply( void )
{
  unsigned char sreg, val_low, val_high;

  // turn off interrupts
  sreg = SREG;
  cli( );

  // use new configuration value
  ConfigValue = ConfigNewValue;

  // demo mode is active if address is FFF
  ConfigDemoActive = ConfigValue == 0x0FFF;

  // put read value as last byte into MAC and IP addresses
  val_low = (unsigned char)ConfigValue;
  val_high = (unsigned char)(ConfigValue >> 8);
  ConfigMac[5] = val_low;
  ConfigMac[4] = (ConfigMac[4] & 0xF0) | val_high;
  ConfigIp[3] = val_low;
  ConfigIp[2] = (ConfigIp[2] & 0xF0) | val_high;

  // restore interrupt state
  SREG = sreg;
}

// initialize
void ConfigInit( void ) // (extern)
{
  // PORTD to input, pull-ups on
  DDRD = 0x00;
  PORTD = 0xFF;
  // upper half of PORTB to input, pull-ups on
  DDRB &= 0x0F;
  PORTB |= 0xF0;

  // initialize configuration to current value from switches
  ConfigNewValue = ConfigRead( );
  ConfigNewCnt = 0;
  ConfigValue = ConfigValue;
  ConfigApply( );
}

// task function to do the work - call from main loop
//  - returns boolean flag if configuration has changed
char ConfigTask( void ) // (extern)
{
  unsigned short value;

  // read new configuration value
  value = ConfigRead( );
  if( value != ConfigNewValue ) // change => re-start counting number of times same value read
    ConfigNewCnt = 0;
  ConfigNewValue = value; // save new value
  if( ConfigNewCnt < ConfigNewCntMax ) // count number of times same value read
    ConfigNewCnt++;

  // configuration value is stable and has changed
  if( ConfigNewCnt >= ConfigNewCntMax && ConfigNewValue != ConfigValue )
  {

    // apply new configuration value
    ConfigApply( );

    // report that configuration changed
    return 1;

  }

  return 0;
}

