/* msDistri - microscope distributor
 * version 0.1 date 2009-06-19
 * Copyright (C) 2009 Stefan Schuermans <stefan@blinkenarea.org>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 * a BlinkenArea project - http://www.blinkenarea.org/
 */

#include <avr/pgmspace.h>
#include "demo.h"
#include "layout.h"
#include "random.h"
#include "serial.h"
#include "status.h"

// number of current demo sequence
static unsigned char DemoAnim = 0;
// number of color (or index, ...) in current demo sequence
static unsigned char DemoColor = 0;

// message to output at begin of demo mode
static unsigned char PROGMEM DemoMessage[] = "Stefan Schuermans, Blinken Area";

// output a single color to all attached pixels
static void DemoOutSingleColor( unsigned char value )
{
  // output MCUF frame with all pixels same color on every output

  // transmit header on every output
  unsigned char data[8] = {
    UPPER_HEIGHT, UPPER_HEIGHT, UPPER_HEIGHT, UPPER_HEIGHT,
    LOWER_HEIGHT, LOWER_HEIGHT, LOWER_HEIGHT, LOWER_HEIGHT
  };
  SerialOutputAll( 0x23 ); // header
  SerialOutputAll( 0x54 );
  SerialOutputAll( 0x26 );
  SerialOutputAll( 0x66 );
  SerialOutputAll( 0x00 ); // height
  SerialOutputEach( data );
  SerialOutputAll( 0x00 ); // width
  SerialOutputAll( 0x08 );
  SerialOutputAll( 0x00 ); // channels
  SerialOutputAll( 0x01 );
  SerialOutputAll( 0x00 ); // maxval
  SerialOutputAll( 0xFF );

  // transmit pixel data
  for( unsigned char i = 0; i < 8 * UPPER_HEIGHT || i < 8 * LOWER_HEIGHT; i++ )
    SerialOutputAll( value );
}

// tick procedure - call every 20ms
void DemoTick20( void ) // (extern)
{
  // exit if demo mode is not active
  if( ! StatusDemoActive )
    return;

  // output current color of current animation and go to next step
  switch( DemoAnim ) {
    // message
    case 0:
      if( DemoColor < sizeof( DemoMessage ) )
        DemoOutSingleColor( pgm_read_byte_near( DemoMessage + DemoColor ) );
      DemoColor++;
      if( DemoColor >= sizeof( DemoMessage ) ) {
        DemoColor = 0;
        DemoAnim = 1;
      }
      break;
    // black -> white
    case 1:
      DemoOutSingleColor( DemoColor );
      DemoColor += 5;
      if( DemoColor >= 255 ) {
        DemoColor = 0;
        DemoAnim = 2;
      }
      break;
    // while -> black
    case 2:
      DemoOutSingleColor( 255 - DemoColor );
      DemoColor += 5;
      if( DemoColor >= 255 ) {
        DemoColor = 0;
        DemoAnim = 3;
      }
      break;
    // end of demo
    default:
      DemoOutSingleColor( 0 );
      DemoColor = 0;
      DemoAnim = 0;
  }
}

