/* msDistri - microscope distributor
 * version 0.1 date 2009-06-19
 * Copyright (C) 2009 Stefan Schuermans <stefan@blinkenarea.org>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 * a BlinkenArea project - http://www.blinkenarea.org/
 */

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/wdt.h>

#include "arp.h"
#include "config.h"
#include "demo.h"
#include "ip.h"
#include "random.h"
#include "rtl8019.h"
#include "serial.h"
#include "status.h"
#include "timing.h"

// main code entry point
int main( void )
{
  wdt_reset( );
  //wdt_disable( );
  wdt_enable( WDTO_60MS );
  wdt_reset( );

  // set up IO-pins of debug interface
  PORTB |= 0x02; // SCK as input with pull-up turned on
  DDRB &= ~0x02;
  PORTE |= 0x03; // MOSI and MISO as input with pull-up turned on
  DDRE &= ~0x03;

  // set up interrupt input from RTL8019AS
  PORTE &= ~0x10;
  DDRE &= ~0x10;

  // set up unused IO-pins as outputs (low)
  PORTE &= ~0x80;
  DDRE |= 0x80;
  PORTG &= ~0x1F; // PG5-7 do not exist
  DDRG |= 0x1F;

  // initialize modules
  ArpInit( );
  ConfigInit( );
  IpInit( );
  RtlInit( );
  SerialInit( );
  StatusInit( );
  TimingInit( );

  // outer main loop
  while( 1 )
  {

    // set up networking stuff with new config
    RtlInit( ); // re-initialize RTL8019
    ArpInit( ); // clear ARP state
    IpInit( ); // clear IP state

    // enable interrupts
    sei( );

    // inner main loop
    while( ! ConfigTask( ) )
    {
      wdt_reset( );
      RandomTask( );
      wdt_reset( );
      RtlTask( );
      wdt_reset( );
      TimingTask( );
      wdt_reset( );
    }

    // disable interrupts
    cli( );

  } // while( 1 )

  return 0;
}

