/* msDistri - microscope distributor
 * version 0.1 date 2009-06-19
 * Copyright (C) 2009 Stefan Schuermans <stefan@blinkenarea.org>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 * a BlinkenArea project - http://www.blinkenarea.org/
 */

#include <string.h>
#include <avr/io.h>

#include "serial.h"

// tick counter
// - decreased in 200ms intervals to disable serial output for some time at startup
static unsigned char SerialTickCntDisable = 15; // 3 seconds

// initialize
void SerialInit( void ) // (extern)
{
  // set up IO pins of serial data outputs as output (high)
  PORTF = 0xFF;
  DDRF = 0xFF;
}

// output a byte on every serial port
//  - data must point to 8 bytes
//  - byte N is output on port N
//  - transmission with 115200bps, 8N1
extern void SerialOutputBytes( unsigned char * data );
// this function is written in assembler, see serial_asm.S

// output the same byte on all serial ports
//  - value: the value to output
void SerialOutputAll( unsigned char value ) // (extern)
{
  // set up data block to output
  unsigned char data[8];
  for( unsigned char n = 0; n < 8; n++ )
    data[n] = value;

  // output data
  if( SerialTickCntDisable <= 0 )
    SerialOutputBytes( data );
}

// output a special byte on one serial port (and annother one on all other ports)
//  - port: the number of the port to output the value on: 0 ... 7
//  - value_one: the value to output on the specified port, 0x00 ... 0xFF
//  - value_other: the value to output on all other ports, 0x00 ... 0xFF
void SerialOutputOne( unsigned char port, unsigned char value_one, unsigned char value_other ) // (extern)
{
  // set up data block to output
  unsigned char data[8];
  for( unsigned char n = 0; n < 8; n++ )
    data[n] = n == port ? value_one : value_other;

  // output data
  if( SerialTickCntDisable <= 0 )
    SerialOutputBytes( data );
}

// output a byte on each serial port
//  - data are the 8 bytes to output on the 8 ports
void SerialOutputEach( unsigned char data[8] ) // (extern)
{
  // output data
  if( SerialTickCntDisable <= 0 )
    SerialOutputBytes( data );
}

// tick procedure - call every 200ms
void SerialTick200( void ) // (extern)
{
  // count down number of steps serial output are disabled
  if( SerialTickCntDisable > 0 )
    SerialTickCntDisable--;
}

