; msDistri - microscope distributor
; version 0.1.1 date 2007-06-19
; Copyright (C) 2009 Stefan Schuermans <stefan@blinkenarea.org>
; Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
; a BlinkenArea project - http://www.blinkenarea.org/

#include <avr/io.h>

.section .text



; data output  0 = PF0
; data output  1 = PF1
; data output  2 = PF2
; data output  3 = PF3
; data output  4 = PF4
; data output  5 = PF5
; data output  6 = PF6
; data output  7 = PF7



; get bit of 
; parameter: bitno = number of bit
; input: Z = pointer to data byte, R16.destbit = 0
; output: Z = pointer to next data byte, R16.destbit = <data byte>.bitno
; changes: R16.destbit, R17
.macro getbit bitno destbit
	ld		R17,Z+
	sbrc		R17,\bitno
	sbr		R16,1<<\destbit
.endm
; 4 cycles



; output a constant bit on all outputs
; parameter: bit = bit to output
; changes: R16, R17
.macro outconstant bit
	rcall		wait34

	ldi		R16,0xFF*\bit
	sts		PORTF,R16

	rcall		wait102
.endm
; 139 cycles



; output one bit on all channels
; parameter: bitno = number of bit
; input: R24:R24 = pointer to 8 data bytes
; changes: R16, R17
.macro outchannels bitno
	mov		ZL,R24			; load data address into Z
	mov		ZH,R25

	clr		R16
	getbit		\bitno,0
	getbit		\bitno,1
	getbit		\bitno,2
	getbit		\bitno,3
	getbit		\bitno,4
	getbit		\bitno,5
	getbit		\bitno,6
	getbit		\bitno,7

	sts		PORTF,R16

	rcall		wait102
.endm
; 139 cycles



; output a byte on every serial port
;  - data must point to 8 bytes
;  - byte N is output on port N
;  - transmission with 115200bps, 8N1
; extern void SerialOutputBytes( unsigned char * data );
.global SerialOutputBytes
.func SerialOutputBytes
SerialOutputBytes:
	push		R16			; save registers
	push		R17
	push		ZL
	push		ZH

	; parameters: R25:R24=data

	; 115200bps, 16MHz -> 139 cycles

	in		R16,SREG-0x20		; interrupts off
	push		R16
	cli

	outconstant	0			; start bit

	outchannels	0			; D0
	outchannels	1			; D1
	outchannels	2			; D2
	outchannels	3			; D3
	outchannels	4			; D4
	outchannels	5			; D5
	outchannels	6			; D6
	outchannels	7			; D7

	outconstant	1			; stop bit

	pop		R16			; restore interrupt state
	out		SREG-0x20,R16

	pop		ZH			; restore registers
	pop		ZL
	pop		R17
	pop		R16
	ret
.endfunc



; wait x cycles (including rcall and ret)
; changes: R16
wait34:
	ldi	R16,9				; 1 cycle, N=9
wait34_loop:
	dec	R16				; N cycles
	brne	wait34_loop			; 2N-1 cycles
	ret					; 7 cycles (including rcall)
wait102:
	nop
	nop
	ldi	R16,31				; 1 cycle, N=31
wait102_loop:
	dec	R16				; N cycles
	brne	wait102_loop			; 2N-1 cycles
	ret					; 7 cycles (including rcall)



