/* msDistri - microscope distributor
 * version 0.1 date 2009-06-19
 * Copyright (C) 2009 Stefan Schuermans <stefan@blinkenarea.org>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 * a BlinkenArea project - http://www.blinkenarea.org/
 */

#include <avr/io.h>

#include "config.h"
#include "demo.h"
#include "status.h"

// number of times to blink status LED
#define StatusBlinkCntSerialData 1
#define StatusBlinkCntTimeout 2
#define StatusBlinkCntDemo 3
unsigned char StatusBlinkCnt = StatusBlinkCntTimeout;

// interval counter for blinking status LED
unsigned char StatusBlinkInterval = 0;

// number 200ms ticks no MCUF data event has been received
#define StatusMcufIdleMax 25 // 5 seconds
unsigned char StatusMcufIdle = StatusMcufIdleMax;

// demo mode active
char StatusDemoActive = 0; // boolean flag if demo mode is active (extern)

// initialize
void StatusInit( void )
{
	//set up IO pin of status LED as output and turn on status LED
	DDRB |= 0x01;
	PORTB |= 0x01;
}

// status event - received MCUF data to output
void StatusEventMcuf( void )
{
	// MCUF is not idle
	StatusMcufIdle = 0;

	// turn off demo mode
	StatusDemoActive = 0;
}

// tick procedure - call every 200ms
void StatusTick200( void )
{
	// count number of 200ms ticks MCUF is idle
	if( StatusMcufIdle < StatusMcufIdleMax )
		StatusMcufIdle++;

	// check if to enable demo mode
	StatusDemoActive = ConfigDemoActive && StatusMcufIdle >= StatusMcufIdleMax;

	// turn out status LED only in even intervals and if not yet blinked StatusBlinkCnt times
	if( (StatusBlinkInterval & 0x01) == 0 && StatusBlinkInterval >> 1 < StatusBlinkCnt )
		PORTB |= 0x01;
	else
		PORTB &= ~0x01;

	// next interval
	StatusBlinkInterval++;

	// blink cycle finished
	if( StatusBlinkInterval >= 10 ) {
		StatusBlinkInterval = 0;

		// set new blink count
		if( StatusMcufIdle < StatusMcufIdleMax ) {
			StatusBlinkCnt = StatusBlinkCntSerialData;
		} else {
			if( StatusDemoActive ) {
				StatusBlinkCnt = StatusBlinkCntDemo;
			} else {
				StatusBlinkCnt = StatusBlinkCntTimeout;
			}
		}
	}
}

